@@ -16,8 +16,10 @@ use offspring_factory::OffspringFactory;
 pub use species::{Species, SpeciesID};
 
 use rand::prelude::Rng;
+use serde::{Deserialize, Serialize};
 
 /// A population of genomes.
+#[derive(Serialize, Deserialize)]
 pub struct Population<C, H, G> {
     species: Vec<Species<G>>,
     history: H,
@@ -33,15 +35,11 @@ where
 {
     /// Creates a new population using the passed configurations.
     ///
-    /// These configurations shouldn't be modified once evolution
-    /// begins, thus they are copied and kept by the population for
-    /// the duration of its lifetime.
-    /// 
     /// The type of `genetic_config` depends on the implementation
     /// of [`Genome`], and is effectively opaque to the population.
     ///
     /// [`Genome`]: crate::Genome
-    /// 
+    ///
     /// # Examples
     /// ```
     /// # use oxineat_nn::genomics::{GeneticConfig, NNGenome as G};
@@ -52,7 +50,7 @@ where
     ///     ..PopulationConfig::zero()
     /// };
     /// # let genetic_config = GeneticConfig::zero();
-    /// 
+    ///
     /// // With `G` a suitable type implementing `Genome`...
     /// let population = Population::<_, _, G>::new(pop_config, genetic_config);
     /// ```
@@ -63,9 +61,8 @@ where
         Population {
             species: {
                 let mut s0 = Species::new(SpeciesID(0, 0), G::new(&genetic_config));
-                s0.genomes.extend(
-                    (1..population_config.size.get()).map(|_| G::new(&genetic_config)),
-                );
+                s0.genomes
+                    .extend((1..population_config.size.get()).map(|_| G::new(&genetic_config)));
                 vec![s0]
             },
             history: H::new(&genetic_config),
@@ -76,6 +73,103 @@ where
         }
     }
 
+    /// Creates a new population using the passed configurations,
+    /// and seeds it with the specified genomes. Each array of genomes
+    /// is assigned to its own species, with the first as the
+    /// species representative. If the number of seed genomes is not
+    /// as large as the configured population size, the remaining
+    /// space is filled as during normal population
+    ///
+    /// Returns `None` if either the configured population size is
+    /// lesser than the number of seed genomes, or any of the genomes
+    /// are incompatible with the specified genetic config, as established
+    /// by [`Genome::conforms_to`].
+    ///
+    /// The type of `genetic_config` depends on the implementation
+    /// of [`Genome`], and is effectively opaque to the population.
+    ///
+    /// [`Genome`]: crate::Genome
+    /// [`Genome::conforms_to`]: crate::Genome::conforms_to
+    ///
+    /// # Examples
+    /// ```
+    /// # use oxineat_nn::genomics::{GeneticConfig, NNGenome};
+    /// use oxineat::{Population, PopulationConfig};
+    ///
+    /// let pop_config = PopulationConfig {
+    ///     // Set desired configuration
+    ///     size: std::num::NonZeroUsize::new(100).unwrap(),
+    ///     ..PopulationConfig::zero()
+    /// };
+    /// # let genetic_config = GeneticConfig {
+    /// #     weight_bound: 1.0,
+    /// #     ..GeneticConfig::zero()
+    /// # };
+    /// # let g1 = NNGenome::new(&genetic_config);
+    /// # let g2 = NNGenome::new(&genetic_config);
+    /// # let g3 = NNGenome::new(&genetic_config);
+    ///
+    /// // With `seed` a slice of a suitable type implementing `Genome`...
+    /// let population = Population::new_seeded(vec![vec![g1, g2], vec![g3]], pop_config, genetic_config).unwrap();
+    ///
+    /// # assert_eq!(population.species().count(), 3);
+    /// # assert_eq!(population.species().map(|s| s.genomes().count()).collect::<Vec<_>>(), vec![97, 2, 1]);
+    /// ```
+    pub fn new_seeded(
+        genomes: Vec<Vec<G>>,
+        population_config: PopulationConfig,
+        genetic_config: C,
+    ) -> Option<Population<C, H, G>>
+    where
+        H: InnovationHistory<Config = C>,
+    {
+        let seed_count = genomes.iter().map(|v| v.len()).sum();
+        if population_config.size.get() < seed_count {
+            return None;
+        }
+
+        if !genomes
+            .iter()
+            .map(|v| v.iter())
+            .flatten()
+            .all(|g| g.conforms_to(&genetic_config))
+        {
+            return None;
+        }
+
+        let species: Vec<Species<G>> = std::iter::once({
+            let mut s0 = Species::new(SpeciesID(0, 0), G::new(&genetic_config));
+            s0.genomes.extend(
+                (1..population_config.size.get() - seed_count).map(|_| G::new(&genetic_config)),
+            );
+            s0
+        })
+        .chain(
+            genomes
+                .into_iter()
+                .filter(|v| !v.is_empty())
+                .enumerate()
+                .map(|(i, mut genomes)| {
+                    let mut s = Species::new(SpeciesID(0, i + 1), genomes.swap_remove(0));
+                    for g in genomes {
+                        s.add_genome(g.clone())
+                    }
+                    s
+                }),
+        )
+        .collect();
+        let species_count = species.len();
+
+        Some(Population {
+            species: species,
+            history: H::new(&genetic_config),
+            generation: 0,
+            historical_species_count: species_count,
+            population_config,
+            genetic_config,
+        })
+    }
+
     /// Evaluates the fitness of each genome in the
     /// population using the passed evaluator.
     ///
@@ -520,9 +614,19 @@ fn round_retain_sum(values: &[f32]) -> Vec<usize> {
 mod tests {
     #[test]
     fn round_retain_sum() {
-        let v = [5.2, 9.5, 2.8, 1.3, 2.2, 2.7, 6.3];
+        let v = [
+            5.2,
+            9.5,
+            2.8,
+            1.3,
+            2.2,
+            2.7,
+            6.3,
+            1.0000000000001,
+            0.9999999999999,
+        ];
         let w = super::round_retain_sum(&v);
         assert_eq!(v.iter().sum::<f32>(), w.iter().sum::<usize>() as f32);
-        assert_eq!(w, [5, 10, 3, 1, 2, 3, 6]);
+        assert_eq!(w, [5, 10, 3, 1, 2, 3, 6, 1, 1]);
     }
 }
